#include <qapplication.h>
#include <qmessagebox.h>
#include <math.h>
#include "cfgdialog.h"
#include "tools.h"
#include "names.h"
#include "color.h"
#include "outcontrol.h"
#include "device.h"

extern CFGTree* tree;
extern DevManager devmanager;
extern OutControl outcontrol;
extern bool isMassRoot;

static const char* YESNO[] = {"Yes","No"};
static int Nyes = 2;

static const char* ONOFF[] = {"On","Off"};
static int Nonoff = 2; 

static const char* OPENCLOSE[] = {"Open","Close"};
static int Nopen = 2; 

static const char* HARA[] = {"HA","RA"};
static int Nhara = 2; 

static const char* METHOD[] = {"ApertureShift","TelescopeShift"};
static int Nmethod = 2; 

static const char* CN2METHOD[] = {"X","L"};
static int Ncn2method = 2; 

static const char* REFER[] = {"FirstMean","LastMean","SeasonMean"};
static int Nrefer = 3;

static const char* PRFTYPE[] = {"Bars","Smooth"};
static int Nprf = 2;

QValidator::State FloatValidator::validate( QString& text, int& pos ) {
	jdebug( "Text: %s len:%d pos: %d",text.data(),text.length(),pos );
	if( pos == 0 ) return Valid;
	if( !text.length() ) {
		jdebug(" !text.length(), valid");
		return Valid;
	}
	if( (unsigned int)pos > text.length() ) {
		jdebug("  >= text.length(), valid");
		return Valid;
	}
/*
	if( (!isdigit(text[pos-1])) && (text[pos-1] != '.')
			&& (text[pos-1] != '-') && (text[pos-1] != '+') ) {
		jdebug( "pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1] );
		return Invalid;
	}
*/
	if( text[pos-1] == '.' ) {
		if( text.contains('.') > 1 ) return Invalid;
	}
	if( (text.find('-') != -1) && (text.find('-') != 0) ) return Invalid;
	if( (text.find('+') != -1) && (text.find('+') != 0) ) return Invalid;

	if( text.contains('-') > 1 ) return Invalid;
	if( text.contains('+') > 1 ) return Invalid;

	if( text.length() < 1 ) {
		jdebug(" Less than five digits; not acceptable" );
		return Valid;
	}
	return Valid;
//	return Acceptable;
}

QValidator::State IntValidator::validate( QString& text, int& pos ) {
	jdebug( "Text: %s len:%d pos: %d",text.data(),text.length(),pos );
	if( pos == 0 ) return Valid;
	if( !text.length() ) {
		jdebug(" !text.length(), valid");
		return Valid;
	}
	if( (unsigned int)pos > text.length() ) {
		jdebug("  >= text.length(), valid");
		return Valid;
	}
/*
	if( (!isdigit(text[pos-1])) && (text[pos-1] != '-')
			&& (text[pos-1] != '+') ) {
		jdebug( "pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1] );
		return Invalid;
	}
*/
	if( (text.find('-') != -1) && (text.find('-') != 0) ) return Invalid;
	if( (text.find('+') != -1) && (text.find('+') != 0) ) return Invalid;

	if( text.contains('-') > 1 ) return Invalid;
	if( text.contains('+') > 1 ) return Invalid;

	return Valid;
}
/*
QValidator::State TimeValidator::validate( QString& text, int& pos ) {
	jdebug( "Text: %s len:%d pos: %d",text.data(),text.length(),pos );
	if( pos == 0 ) return Valid;
	if( !text.length() ) {
		jdebug(" !text.length(), valid");
		return Valid;
	}
	if( (unsigned int)pos > text.length() ) {
		jdebug("  >= text.length(), valid");
		return Valid;
	}

	if( (!isdigit(text[pos-1])) && (text[pos-1] != '.') && (text[pos-1] != ' ') ) {
		jdebug( "pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1] );
		return Invalid;
	}

	int b1 =  text.find(' ');
	int b2 = -1;
	jdebug( "b1=%d len=%d",b1,text.length() );	
	if( text.length() > b1 ) b2 = text.find(' ',b1+1);
	jdebug("b2=%d",b2);
	if( text[pos-1] == ' ' ) {
		if( (b1 != 1) && (b1 != 2) ) return Invalid;
		if( b2 != -1 ) {
			if( text.find(' ',b2+1) != -1 ) return Invalid;
		}
	}
//	if( atoi(&text[0]) >= 24 ) return Invalid;
	if( (b1 != -1) && ((text.length()-1) > b1) )
		if( atoi(&text[b1+1]) > 59 ) return Invalid;
	if( (b2 != -1) && ((text.length()-1) > b2) )
		if( atof(&text[b2+1]) >= 60 ) return Invalid;

	if( (text.find('-') != -1) && (text.find('-') != 0) ) return Invalid;
	if( (text.find('+') != -1) && (text.find('+') != 0) ) return Invalid;

	if( text.contains('-') > 1 ) return Invalid;
	if( text.contains('+') > 1 ) return Invalid;

	if( (text[pos-1] == '.') && ((b2 == -1) || ((pos-1) < b2)) ) return Invalid;
	if( (text[pos-1] == '.') && (text.contains('.') > 1) ) return Invalid;

	jdebug( "Input Valid" );
	return Valid;
}
*/
/*
QValidator::State DegreeValidator::validate( QString& text, int& pos ) {
	jdebug( "Text: %s len:%d pos: %d",text.data(),text.length(),pos );
	if( pos == 0 ) return Valid;
	if( !text.length() ) {
		jdebug(" !text.length(), valid");
		return Valid;
	}
	if( (unsigned int)pos > text.length() ) {
		jdebug("  >= text.length(), valid");
		return Valid;
	}

	if( (!isdigit(text[pos-1])) && (text[pos-1] != '.') && 
			(text[pos-1] != ' ') && (text[pos-1] != '-') &&
										(text[pos-1] != '+') ) {
		jdebug( "pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1] );
		return Invalid;
	}

	int b1 =  text.find(' ');
	int b2 = -1;
	jdebug( "b1=%d len=%d",b1,text.length() );	
	if( text.length() > b1 ) b2 = text.find(' ',b1+1);
	jdebug("b2=%d",b2);
	if( text[pos-1] == ' ' ) {
		if( (b1 != 1) && (b1 != 2) && (b1 != 3) ) return Invalid;
		if( b2 != -1 ) {
			if( text.find(' ',b2+1) != -1 ) return Invalid;
		}
	}
//	jdebug( "%d",atoi(&text[0]) );
//	if( abs(atoi(&text[0])) > 90 ) return Invalid;
	if( (b1 != -1) && ((text.length()-1) > b1) ){
		if( atoi(&text[b1+1]) > 59 ) return Invalid;
		if( (abs(atoi(&text[0])) == 90) && (atoi(&text[b1+1]) > 0) ) return Invalid;
	}
	if( (b2 != -1) && ((text.length()-1) > b2) ){
		if( atof(&text[b2+1]) >= 60 ) return Invalid;
		if( (abs(atoi(&text[0])) == 90) && (atof(&text[b2+1]) > 0) ) return Invalid;
	}
	if( (text.find('-') != -1) && (text.find('-') != 0) ) return Invalid;
	if( (text.find('+') != -1) && (text.find('+') != 0) ) return Invalid;

	if( text.contains('-') > 1 ) return Invalid;
	if( text.contains('+') > 1 ) return Invalid;

	if( (text[pos-1] == '.') && ((b2 == -1) || ((pos-1) < b2)) ) return Invalid;
	if( (text[pos-1] == '.') && (text.contains('.') > 1) ) return Invalid;

	jdebug( "Input Valid" );
	return Valid;
}
*/

SubItem::SubItem( CFGBranch* _branch, CFGBranch* _bak_branch ) {
	_enable = true;
	branch = _branch;
	bak_branch = _bak_branch;
}

SubItem::~SubItem() {
}

void SubItem::setEnabled( void ) {
	_enable = true;
}

void SubItem::setDisabled( void ) {
	_enable = false;
}

bool SubItem::searchChangeDisabledLeafs( void ) {
	bool emitSignalChanged = false;
	branch->toFirst();
	while( branch->getCurrentLink() ) {
		CFGLeaf* leaf =  branch->getCurrentLink()->getLeaf();
		if( leaf && leaf->isChanged()
			&&!(leaf->isChangeEnabled()) ) {
			leaf->setChanged( false );
			QString text;
			text.sprintf( "Field: %s Active after restarting",leaf->getKey() );
			QMessageBox::warning( 0,"cfg",text.data() );
			jdebug( "leaf: %s - change disabled!!!", leaf->getKey() );
		}
		if( leaf && leaf->isChanged() ) emitSignalChanged = true;
		branch->toNext();
	}
	return emitSignalChanged;
}

void SubItem::slotExecBranchDialog( void ) {

	bak_branch->copyBranch( bak_branch,branch );
	BranchDialog* dialog = new BranchDialog( branch, bak_branch, _enable );
	CHECK_PTR(dialog);
	connect( dialog,SIGNAL(signalShowStatus(const char*,int)),
				this,SIGNAL(signalShowStatus(const char*,int)) );

	int ret = dialog->exec();

	if( ret == QDialog::Accepted ) {
		bool emitSignalChanged = searchChangeDisabledLeafs();

		branch->copyBranch( branch,bak_branch );
		tree->setChanged(1);
		emit signalShowStatus( "Accepted",2000 );
		if( emitSignalChanged ) tree->emitSignalChanged();
	}
	delete dialog;
}

bool calcAccess	( bool _enable, bool for_root, bool count_disabled ) {
	bool access = (_enable || !count_disabled) && 
		!(for_root && !isMassRoot); // enable only for mass_root
	return access;
}

JKQLineEdit::JKQLineEdit( QWidget* parent, CFGLeaf* _leaf, bool _enable ) 
						: QLineEdit( parent ) {
		printf("JKQLineEdit::JKQLineEdit leaf=|%s|\n",_leaf->getData()->data());
	leaf = _leaf;
	_enable = calcAccess( _enable,leaf->isForRoot(),
				leaf->isCountDisabled() );
	setEnabled( _enable );
}

JKQLineEdit::~JKQLineEdit() {
}

/*
void warningAboutChangeDisabledLeaf( CFGLeaf* leaf ) {
	QString text;
	text.sprintf( "Field: %s Active after restarting",leaf->getKey() );
	QMessageBox::warning( 0,"cfg",text.data() );
}
*/

void JKQLineEdit::slotTextChanged( const QString& _text ) {
	jdebug( "textChanged: %s", _text.data() );
	leaf->replaceData( _text );
//	if( leaf->isChangeEnabled() ) {
	leaf->setChanged( true );
//	} else {
//		warningAboutChangeDisabledLeaf( leaf );
//	}
}

void JKQLineEdit::restore( void ) {
	jdebug( "restore" );
	setText( getLeafText() );
	leaf->setChanged( false );
}

void JKQLineEdit::enterEvent( QEvent* e ) {
	emit signalShowStatus( leaf->getHelp()->data(),-1 );
	QLineEdit::enterEvent( e );
}

void JKQLineEdit::leaveEvent( QEvent* e ) {
	emit signalShowStatus( "", 0 );
	QLineEdit::leaveEvent( e );
}

JKQComboBox::JKQComboBox( bool rw, QWidget* parent,CFGLeaf* _leaf,
				bool _enable ) : QComboBox( rw,parent ) {
	leaf = _leaf;
	_enable = calcAccess( _enable,leaf->isForRoot(),
				leaf->isCountDisabled() );
	setEnabled( _enable );
}

JKQComboBox::~JKQComboBox() {
}

void JKQComboBox::setCurrentText( const char* _text ) {

	for( int i=0;i<count() ;i++ ) {
		if( !strcasecmp(text(i),_text) ) {
			setCurrentItem( i );
		}
	}
}

void JKQComboBox::slotTextChanged( const QString& _text ) {
	jdebug( "textChanged: %s", _text.data() );
	leaf->replaceData( _text );
//	if( leaf->isChangeEnabled() ) {
	leaf->setChanged( true );
//	} else {
//		warningAboutChangeDisabledLeaf( leaf );
//	}
}

void JKQComboBox::restore( void ) {
	jdebug( "restore" );
	setCurrentText( getLeafText() );
	leaf->setChanged( false );
}

void JKQComboBox::enterEvent( QEvent* e ) {
	emit signalShowStatus( leaf->getHelp()->data(),-1 );
	QComboBox::enterEvent( e );
}

void JKQComboBox::leaveEvent( QEvent* e ) {
	emit signalShowStatus( "", 0 );
	QComboBox::leaveEvent( e );
}

BranchDialog::BranchDialog( CFGBranch* _branch, CFGBranch* _bak_branch, bool _enable ) :
		QDialog( 0,"BranchDialog",true /*modal*/ ) {

//	printf("1 _branch |%s|\n",_branch->getName());
//	_branch->printBranch(_branch);



	bak_branch = _bak_branch;
	branch = _branch;
//	int layout_row;
	int num_layout_cols=2;
//	int w_height = 0;

// lll
//	setSizeGripEnabled(true);
//	setFont( QFont("Helvetica",12),true );

	setCaption( "Section Dialog" );

	okbutton = new QPushButton( "Apply",this );
	CHECK_PTR(okbutton);
	okbutton->setAutoDefault( true );
	connect( okbutton, SIGNAL( clicked() ),
				this, SLOT( accept() ) );

	restorebutton = new QPushButton( "Restore",this );
	CHECK_PTR(restorebutton);
	restorebutton->setAutoDefault( true );
	connect( restorebutton, SIGNAL( clicked() ),
				this, SLOT( slotRestored() ) );

	cancelbutton = new QPushButton( "Cancel",this );
	CHECK_PTR(cancelbutton);
	cancelbutton->setAutoDefault( true );
	connect( cancelbutton, SIGNAL( clicked() ),
				this, SLOT( reject() ) );

	int border = fontMetrics().height();
	int labelheight =  fontMetrics().height()*5/4 ;
	int row=0;
	branch->toFirst();
	while( branch->getCurrentLink() ) {
		row++;
		branch->toNext();
	}

	int w_height = (labelheight+border) * ( row+3 ) + border; 
	if( w_height > qApp->desktop()->height() ) {
		int maxWindowFontHeight = int(floor(qApp->desktop()->height() / ( 2.25*(row+3) + 1)));
		int newFontSize = maxWindowFontHeight - 1;	// empirical, size != height !!!
		const int minFontSize = 8;
		while( newFontSize > minFontSize ) {
			QFont font = QFont("Helvetica",newFontSize);
// lll
//				printf("newFontSize = %d\n",newFontSize);
//				if(font.exactMatch()) printf("is exactMatch!\n");				
			if( font.exactMatch()
				&& (QFontMetrics(font).height() <= maxWindowFontHeight) ) break;
			newFontSize--;
		}
		setFont( QFont("Helvetica",newFontSize),true );
//				printf("newFontSize = %d, okonchatel'no\n",newFontSize);
		border = fontMetrics().height();
		labelheight =  fontMetrics().height()*5/4 ;
	}

	layout = new QGridLayout( this,1,num_layout_cols,border );
	CHECK_PTR(layout);
	int layout_row = 1;

	int labelwidth = 0;
	int editwidth = 0;
	bool enableApply = false;
	int maxlabelwidth = okbutton->sizeHint().width();
	int maxeditwidth = cancelbutton->sizeHint().width();

	branch->toFirst();
	while( branch->getCurrentLink() ) {
		if( branch->getCurrentLink()->getId() == 'l' ) {
			Link* link = branch->getCurrentLink();
			labelwidth = fontMetrics().width( link->getLeafKey() );
			editwidth = fontMetrics().width( link->getLeafData() );
			if( labelwidth > maxlabelwidth ) maxlabelwidth = labelwidth;
			if( editwidth > maxeditwidth ) maxeditwidth = editwidth;
			enableApply = enableApply ||
					(_enable ||
				!(link->getLeaf()->isCountDisabled()));
		}
		branch->toNext();
	}

	okbutton->setEnabled( enableApply );
	restorebutton->setEnabled( enableApply );

	maxeditwidth += fontMetrics().width( "ww" ); //  ...
	if( maxeditwidth+maxlabelwidth < fontMetrics().width(branch->getName()) ) {
		int dwidth = ( fontMetrics().width(branch->getName())-
							(maxeditwidth+maxlabelwidth) )/2; 
		maxeditwidth += dwidth;
		maxlabelwidth += dwidth;
	}

	int halfDesk =  qApp->desktop()->width()/2;
	if( maxeditwidth > halfDesk ) maxeditwidth = halfDesk;
	if( maxlabelwidth > halfDesk ) maxlabelwidth = halfDesk;

//	int labelheight =  fontMetrics().height()*5/4 ;

	n_edit = 0;
	n_combo = 0;
	n_label = 0;
	edit.resize(10);
	combo.resize(10);
	label.resize(10);
	label[n_label] = new QLabel( branch->getName(),this );
	CHECK_PTR(label[n_label]);
	label[n_label]->setFixedSize( label[n_label]->sizeHint() );
	layout->addMultiCellWidget( label[n_label],0,0,0,1,AlignHCenter );	
	n_label++;

//	floatvalidator = new FloatValidator(this);
//	CHECK_PTR(floatvalidator);
//	timevalidator = new TimeValidator(this);
//	CHECK_PTR(timevalidator);
//	degreevalidator = new DegreeValidator(this);
//	CHECK_PTR(degreevalidator);

//	printf("2 branch |%s|\n",branch->getName());
//	branch->printBranch(branch);

/* */
	while( branch->getCurrentLink() ) {
		if( branch->getCurrentLink()->getId() == 'l' ) {
					printf("id = l |%s|\n",branch->getCurrentLink()->getLeafData());

			layout->expand( layout_row+1,num_layout_cols );

			if( (int)label.size() < n_label+1 ) label.resize(label.size()+10);
			label[n_label] = new QLabel( branch->getCurrentLink()->getLeafKey(),this );
			CHECK_PTR(label[n_label]);
			label[n_label]->setFixedSize( maxlabelwidth,labelheight );
			layout->addWidget( label[n_label],layout_row,0 );
			n_label++;

			const char* validat = branch->getCurrentLink()->getLeaf()->getValidatorType();
			if( !validat )	validat = "empty";


			if( !strcasecmp( validat,"yes" )        ||
				!strcasecmp( validat,"onoff" )  ||
				!strcasecmp( validat,"onoff" )  ||
				!strcasecmp( validat,"open" )   ||
				!strcasecmp( validat,"hara" )   ||
				!strcasecmp( validat,"color" )  ||
				!strcasecmp( validat,"method" )  ||
				!strcasecmp( validat,"cn2method" )  ||
				!strcasecmp( validat,"apert" )  ||
				!strcasecmp( validat,"prftype" )||
				!strcasecmp( validat,"lut" )    ||
				!strcasecmp( validat,"refer" ) ) {

				if( (int)combo.size() < n_combo+1 ) combo.resize(combo.size()+10);
				combo[n_combo] = new JKQComboBox(false,this,
						branch->getCurrentLink()->getLeaf(),_enable );
				CHECK_PTR(combo[n_combo]);

				if( !strcasecmp( validat,"yes" ) )
					combo[n_combo]->insertStrList( YESNO,Nyes );
				else if( !strcasecmp( validat,"onoff" ) )
					combo[n_combo]->insertStrList( ONOFF,Nonoff );
				else if( !strcasecmp( validat,"open" ) )
					combo[n_combo]->insertStrList( OPENCLOSE,Nopen );
				else if( !strcasecmp( validat,"hara" ) )
					combo[n_combo]->insertStrList( HARA,Nhara );
				else if( !strcasecmp( validat,"method" ) )
					combo[n_combo]->insertStrList( METHOD,Nmethod );
				else if( !strcasecmp( validat,"cn2method" ) )
					combo[n_combo]->insertStrList( CN2METHOD,Ncn2method );
				else if( !strcasecmp( validat,"color" ) )
					combo[n_combo]->insertStrList( JCOLORS,Ncolors );
				else if( !strcasecmp( validat,"refer" ) )
					combo[n_combo]->insertStrList( REFER,Nrefer );
				else if( !strcasecmp( validat,"prftype" ) )
					combo[n_combo]->insertStrList( PRFTYPE,Nprf );
				else if( !strcasecmp( validat,"lut" ) ) {
					int nl = outcontrol.getNlut();
					for( int i=0; i<nl; i++ ) {
						combo[n_combo]->
							insertItem( outcontrol.getLutName(i) );
					}
				} else if( !strcasecmp( validat,"apert" ) ) {
					int ns = devmanager.getNStops();
					for( int i=0; i<ns; i++ ) {
						combo[n_combo]->
							insertItem( devmanager.getStopName(i) );
					}
				}

				combo[n_combo]->setFixedSize( maxeditwidth,labelheight );
				combo[n_combo]->setCurrentText( branch->getCurrentLink()->getLeafData() );
				connect( combo[n_combo],SIGNAL(activated(const QString&)),						
							combo[n_combo],SLOT(slotTextChanged(const QString&)) );
				connect( combo[n_combo],SIGNAL(signalShowStatus(const char*,int)),
							this,SIGNAL(signalShowStatus(const char*,int)) );
				layout->addWidget( combo[n_combo],layout_row,1 );
				n_combo++;

			} else {	
				if( (int)edit.size() < n_edit+1 ) edit.resize(edit.size()+10);
					printf("new JKQLineEdit |%s|\n",branch->getCurrentLink()->getLeafData());
				edit[n_edit] = new JKQLineEdit( this,
						branch->getCurrentLink()->getLeaf(),_enable );
				CHECK_PTR(edit[n_edit]);
				edit[n_edit]->setFixedSize( maxeditwidth,labelheight );
				edit[n_edit]->setText( branch->getCurrentLink()->getLeafData() );

				if( !strcasecmp( validat,"float" ) ) {
//					edit[n_edit]->setValidator( floatvalidator );
				} else if( !strcasecmp( validat,"time" ) ) {
//					edit[n_edit]->setValidator( timevalidator );
				} else if ( !strcasecmp( validat,"degree") ) {
//					edit[n_edit]->setValidator( degreevalidator );
				}
				connect( edit[n_edit],SIGNAL( textChanged(const QString&) ),
						edit[n_edit],SLOT( slotTextChanged(const QString&) ) );
				connect( edit[n_edit],SIGNAL(signalShowStatus(const char*,int)),
							this,SIGNAL(signalShowStatus(const char*,int)) );
				layout->addWidget( edit[n_edit],layout_row,1 );
				n_edit++;
			}
			layout_row++;		

		}
		branch->toNext();
	}
/* */

//	n_label--; n_edit--;n_combo--;
	layout->expand( layout_row+1,num_layout_cols );

	okbutton->setMinimumSize( okbutton->sizeHint() );
	okbutton->setMaximumSize( maxlabelwidth,okbutton->sizeHint().height() );
	layout->addWidget( okbutton,layout_row,0 );	

	cancelbutton->setMinimumSize( cancelbutton->sizeHint() );
	cancelbutton->setMaximumSize( maxeditwidth,cancelbutton->sizeHint().height() );
	layout->addWidget( cancelbutton,layout_row,1 );	

	layout_row++;
	layout->expand( layout_row+1,num_layout_cols );
	restorebutton->setMinimumSize( restorebutton->sizeHint() );
	restorebutton->setMaximumSize( maxlabelwidth,restorebutton->sizeHint().height() );
	layout->addMultiCellWidget( restorebutton,layout_row,layout_row,0,1,AlignCenter );	


	w_height = (labelheight+border) * ( layout_row+1 ) + border; 
	move( ( qApp->desktop()->width()-width() )/2,
					( qApp->desktop()->height()-w_height )/2 );
	emit signalShowStatus( "Section dialog",4000 );
}

void BranchDialog::slotRestored() {
	branch->copyBranch( bak_branch,branch );
	for( int i=0;i<n_edit;i++ ) {
		edit[i]->restore();
//		edit[i]->setText( edit[i]->getLeafText() );
	}
	for( int i=0;i<n_combo;i++ ) {
		combo[i]->restore();
//		combo[i]->setCurrentText( combo[i]->getLeafText() );
	}
	emit signalShowStatus( "Restored",2000 );
}

BranchDialog::~BranchDialog() {
	delete layout;
	delete cancelbutton;
	delete okbutton;
	delete restorebutton;
	int i;
	for( i=0;i<n_label;i++ ) {
		delete label[i];
	}
	for( i=0;i<n_edit;i++ ) {
		delete edit[i];
	}
	for( i=0;i<n_combo;i++ ) {
		delete combo[i];
	}

//	delete floatvalidator;
//	delete timevalidator;
//	delete degreevalidator;

	jdebug( "BranchDialog: Destructor \n" );
}

CFGDialog::CFGDialog( QPopupMenu* menu, int* _item_id,
					QArray<QString*> menu_help ) {
	CFGBranch *branch, *s_branch, *s_s_branch, *bak_branch;

	jdebug( "CFG Dalog: start of Constructor" ); 
//	work_tree = _maintree.linkMainTree();
	work_tree = tree;

	n_section_item = -1;
	n_s_section_item = -1;
	n_sub_item = -1;
	int item_id = *_item_id;

	section_item.resize(10);
	s_section_item.resize(10);
	sub_item.resize(10);

	work_tree->toFirst();
	while( work_tree->getCurrentLink() ) {
		if( work_tree->getCurrentLink()->getId() == 'b' ) {
			branch = work_tree->getCurrentLink()->getBranch();
			if( branch->getNSubBranch() ) { // ..,     =>   
				n_section_item++;
				if( (int)section_item.size() < n_section_item+1 ) 
					section_item.resize(section_item.size()+10);

				section_item[n_section_item] = new QPopupMenu;
				CHECK_PTR( section_item[n_section_item] );	
				menu->insertItem( branch->getName(),section_item[n_section_item],item_id );
				if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
				menu_help[item_id] = new QString( branch->getHelp()->data() );
				item_id++;

				branch->toFirst();
				while( branch->getCurrentLink() ) {
					if( branch->getCurrentLink()->getId() == 'b' ) {
						s_branch = branch->getCurrentLink()->getBranch();
						if( s_branch->getNSubBranch() ) {	//    s_branch
							n_s_section_item++;
							if( (int)s_section_item.size() < n_s_section_item+1 ) 
								s_section_item.resize(s_section_item.size()+10);
							s_section_item[n_s_section_item] = new QPopupMenu;
							CHECK_PTR( s_section_item[n_s_section_item] );	
							section_item[n_section_item]->insertItem( s_branch->getName(),
								s_section_item[n_s_section_item],item_id );							
							if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
							menu_help[item_id] = new QString( s_branch->getHelp()->data() );
							item_id++;

							s_branch->toFirst();
							while( s_branch->getCurrentLink() ) {
								if( s_branch->getCurrentLink()->getId() == 'b' ) {
									s_s_branch = s_branch->getCurrentLink()->getBranch();
									if( !(s_s_branch->getNSubBranch()) ) { //         
										n_sub_item++;
										if( (int)sub_item.size() < n_sub_item+1 ) 
											sub_item.resize(sub_item.size()+10);
										bak_branch = tree->findBranch( branch->getName() )->
															findBranch( s_branch->getName() )->
																findBranch( s_s_branch->getName() );
										sub_item[n_sub_item] = new SubItem( s_s_branch, bak_branch );
										CHECK_PTR(sub_item[n_sub_item]);
										connect( sub_item[n_sub_item],SIGNAL(signalShowStatus(const char*,int)),
											this,SIGNAL(signalShowStatus(const char*,int)) );
//										connect( sub_item[n_sub_item],SIGNAL(signalApply()),
//											this,SLOT(slotApply()) );
										s_section_item[n_s_section_item]->insertItem( s_s_branch->getName(),sub_item[n_sub_item],SLOT(slotExecBranchDialog()),0,item_id ); //  
										if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
										menu_help[item_id] = new QString( s_s_branch->getHelp()->data() );
										item_id++;
									}																		
								}
								s_branch->toNext();
							}
						} else {	//     s_branch =>  
							n_sub_item++;
							if( (int)sub_item.size() < n_sub_item+1 ) 
									sub_item.resize(sub_item.size()+10);
							bak_branch = tree->findBranch( branch->getName() )->findBranch( s_branch->getName() );
							sub_item[n_sub_item] = new SubItem( s_branch, bak_branch );
							CHECK_PTR(sub_item[n_sub_item]);
							connect( sub_item[n_sub_item],SIGNAL(signalShowStatus(const char*,int)),
								this,SIGNAL(signalShowStatus(const char*,int)) );
//							connect( sub_item[n_sub_item],SIGNAL(signalApply()),
//											this,SLOT(slotApply()) );
							section_item[n_section_item]->insertItem( s_branch->getName(),
								sub_item[n_sub_item],SLOT(slotExecBranchDialog()),0,item_id );
							if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
							menu_help[item_id] = new QString( s_branch->getHelp()->data() );
							item_id++;
						}
					}
					branch->toNext();
				}
			} else {									//				 
				n_sub_item++;
				if( (int)sub_item.size() < n_sub_item+1 ) 
					sub_item.resize(sub_item.size()+10);
				bak_branch = tree->findBranch( branch->getName() );
				sub_item[n_sub_item] = new SubItem( branch, bak_branch );
				CHECK_PTR(sub_item[n_sub_item]);
				connect( sub_item[n_sub_item],SIGNAL(signalShowStatus(const char*,int)),
					this,SIGNAL(signalShowStatus(const char*,int)) );
//				connect( sub_item[n_sub_item],SIGNAL(signalApply()),
//								this,SLOT(slotApply()) );
				menu->insertItem( branch->getName(),
					sub_item[n_sub_item],SLOT(slotExecBranchDialog()),0,item_id );
				if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
				menu_help[item_id] = new QString( branch->getHelp()->data() );
				item_id++;
			}
		}
		work_tree->toNext();
	}
	menu->insertSeparator();
	menu->insertItem( "Save CFG",this,SLOT(slotSaveFile()),0,item_id );
	if( (int)menu_help.size() <= item_id ) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString( "Save CFG" );
	item_id++;
	*_item_id = item_id;

//	tree->saved();
//	connect( tree, SIGNAL(changed()),
//				this, SLOT(slotCfgChanged()) );
	jdebug( "CFG Dalog: end of Constructor" ); 
}

CFGDialog::~CFGDialog() {
	for( int i=0;i<=n_section_item;i++ ) {
		delete section_item[i];
	}
	for( int i=0;i<=n_s_section_item;i++ ) {
		delete s_section_item[i];
	}
	for( int i=0;i<=n_sub_item;i++ ) {
		delete sub_item[i];
	}
	jdebug( "CFG Dialog: Destructor" );
}

//void CFGDialog::slotCfgChanged( void ) {
//	_cfgChanged = 1;
//}
//
void CFGDialog::setEnabled( void ) {
	int i;
	for( i=0;i<=n_sub_item;i++ ) {
		sub_item[i]->setEnabled();
	}
}

void CFGDialog::setDisabled( void ) {
	int i;
	for( i=0;i<=n_sub_item;i++ ) {
		sub_item[i]->setDisabled();
	}
}

void CFGDialog::testCfgChang( void ) {
	if( !tree->isChanged() ) return;
	slotSaveFile();
}

void CFGDialog::slotSaveFile( void ) {
	if( !tree->isChanged() ) {
		emit signalShowStatus( "CFG-file not changed so no update needed",2000 );
		return;
	}

	QMessageBox mb( "Warning","Save changes to CFG-file?",
		QMessageBox::Warning,QMessageBox::Yes,
		QMessageBox::No, 0 );
	int ret = mb.exec();
	if( ret != QMessageBox::Yes ) {
		emit signalShowStatus( "CFG-file not saved",2000 );
		return;
	}

	QString backCfg = cfgFile;
	backCfg.append( ".back" );
	rename( cfgFile,backCfg.data() );
	ret = tree->saveToFile( cfgFile );
	
	if( ret == 0 ) emit signalShowStatus( "CFG-file saved",2000 );
	else emit signalShowStatus( "CFG-file not saved",2000 );
}
