/*--------------------------------------------------------------------------*/
/** @file cfgdialog.h
@brief  Module for implemention of the GUI-dialogs of a user with a tree of CFG
parameters.

                  MASS project: TURBINA module file
                       header file for cfgdialog.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef CFGDIALOG_H
#define CFGDIALOG_H

//#include <qapplication.h>
#include <qdialog.h>
#include <qpushbutton.h>
#include <qpopupmenu.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qstring.h>
#include <qlayout.h>
#include <qarray.h>
#include <qvalidator.h>
#include <qcombobox.h>

#include "tree.h"
//#include "maintree.h"

class FloatValidator : public QValidator {
public:
	FloatValidator(QWidget* parent) : QValidator(parent) {};
	virtual State validate( QString&,int& );
};

class IntValidator : public QValidator {
public:
	IntValidator(QWidget* parent) : QValidator(parent) {};
	virtual State validate( QString&,int& );
};

class TimeValidator : public QValidator {
public:
	TimeValidator(QWidget* parent) : QValidator(parent) {};
	virtual State validate( QString&,int& );
};

class DegreeValidator : public QValidator {
public:
	DegreeValidator(QWidget* parent) : QValidator(parent) {};
	virtual State validate( QString&,int& );
};

class SubItem : public QObject {	//    , 
	Q_OBJECT						//     branch_dialog  

private:
	CFGBranch *branch, *bak_branch;
	bool _enable;

public:
	SubItem( CFGBranch*, CFGBranch* );
	~SubItem();
	void setEnabled( void );
	void setDisabled( void );
	bool searchChangeDisabledLeafs( void );

private slots:
	void slotExecBranchDialog( void );
signals:
//	void signalApply( void );
	void signalShowStatus( const char*,int );
}; 

class JKQLineEdit : public QLineEdit {
	Q_OBJECT

public:
	JKQLineEdit( QWidget*, CFGLeaf*, bool );
	~JKQLineEdit();
	const char* getLeafText( void ) { return leaf->getData()->data(); }
	void restore( void );

private: 
	CFGLeaf* leaf;


protected:
	void enterEvent( QEvent* );
	void leaveEvent( QEvent* );

public slots:
	void slotTextChanged( const QString& );

signals:
	void signalShowStatus( const char*,int );
};

class JKQComboBox : public QComboBox {
	Q_OBJECT

public:
	JKQComboBox( bool, QWidget*, CFGLeaf*, bool );
	~JKQComboBox();
	void setCurrentText( const char* );
	const char* getLeafText( void ) { return leaf->getData()->data(); }
	void restore( void );

private: 
	CFGLeaf* leaf;

protected:
	void enterEvent( QEvent* );
	void leaveEvent( QEvent* );

public slots:
	void slotTextChanged( const QString& );

signals:
	void signalShowStatus( const char*,int );
};


class BranchDialog : public QDialog {
	Q_OBJECT

public:
	BranchDialog( CFGBranch*, CFGBranch*, bool ); 
	 ~BranchDialog();
private:

	CFGBranch* branch;
	CFGBranch* bak_branch;	// bak-copy    "Restore"
	
	QPushButton* cancelbutton;
	QPushButton* okbutton;
	QPushButton* restorebutton;
	QGridLayout* layout;

	QArray<QLabel*> label;
	int n_label;
	QArray<JKQLineEdit*> edit;
	int n_edit;
	QArray<JKQComboBox*> combo;
	int n_combo;

	FloatValidator* floatvalidator;	
	TimeValidator* timevalidator;
	DegreeValidator* degreevalidator;

public slots:
	void slotRestored( void );

signals:
	void signalShowStatus( const char*,int );
};

class CFGDialog : public QObject
{
	Q_OBJECT
public:
	CFGDialog( QPopupMenu*,int*,QArray<QString*> );
	~CFGDialog();	
	void setEnabled( void );
	void setDisabled( void );
	void testCfgChang( void );

private:
//	MainTree _maintree; 
	CFGTree *work_tree;

	QArray<QPopupMenu*> section_item,s_section_item;
	int n_section_item,n_s_section_item;

	QArray<SubItem*> sub_item;
	int n_sub_item;

public slots:
	void slotSaveFile( void );

signals:
	void signalShowStatus( const char*,int );
};

#endif
