#include <stdlib.h>	// To prevent problems with missing built-in abs():
#include <math.h>
#include "procounter.h"
#include "tools.h"
#include "names.h"
#include "exchange.h"
#include "cfg.h"

BlockCounter::BlockCounter() {
	_number=0;
}

int BlockCounter::set(int n) {
	if( (n < 0) || (n > 0xffff) ) return FAIL;
	_number = n;
	return SUCCESS;
}

int BlockCounter::incr(void) {
	if(_number >= 0xffff) _number = 0;
	else _number++;
	return _number;
}

int BlockCounter::decr(void) {
	if(_number <= 0) _number = 0xffff;
	else _number--;
	return _number;
}

ProCounter::ProCounter(void) {
	jdebug("ProCounter::ProCounter"); 
	_name = "ProCounter";
	(char*)_identName = NULL;
	(char*)_identSection = NULL;
	resetVariables();
	jdebug("end of ProCounter's constructor");
}

ProCounter::~ProCounter() {
	jdebug("Destructor of ProCounter");
	if(_identName) delete (char*)_identName;
	if(_identSection) delete (char*)_identSection;
	jdebug("End of destructor of ProCounter");
}

void ProCounter::resetVariables(void) {
	jdebug("ProCounter::resetVariables");
	_expositionNumber = UNDEFINED;
	_exposValue = UNDEFINED;
	_blockSize = UNDEFINED;
	_maxNData = 0;
	blockCounter.reset();
	jdebug("End of ProCounter's resetVariables");
}

void ProCounter::create(int sizeOfData) {
	_maxNData = sizeOfData;
	_nBlockLabel = 0;
	_blockSize = 0;
	blockLabel.resize(100);
}

int ProCounter::loadOwnConst(void) {
	jdebug("ProCounter::loadOwnConst");
	_activeOn	= leafDataToChar( deviceCfgFile,_name,"Command","ActiveOn" );
	_activeOff	= leafDataToChar( deviceCfgFile,_name,"Command","ActiveOff" );
	_runMst		= leafDataToChar( deviceCfgFile,_name,"Command","Run" );
	_stop		= leafDataToChar( deviceCfgFile,_name,"Command","Stop" );
	_runTest	= leafDataToChar( deviceCfgFile,_name,"Command","RunTest" );
	_getNumber	= leafDataToChar( deviceCfgFile,_name,"Command","GetNumber" );
	_setNumber	= leafDataToChar( deviceCfgFile,_name,"Command","SetNumber" );
	_getBlSize	= leafDataToChar( deviceCfgFile,_name,"Command","GetBlSize" );
	_setBlSize	= leafDataToChar( deviceCfgFile,_name,"Command","SetBlSize" );
	_setExpos	= leafDataToChar( deviceCfgFile,_name,"Command","SetExpos" );
	_getExpos	= leafDataToChar( deviceCfgFile,_name,"Command","GetExpos" );
	_masterOn	= leafDataToChar( deviceCfgFile,_name,"Command","MasterOn" );
	_masterOff	= leafDataToChar( deviceCfgFile,_name,"Command","MasterOff" );
	_induceOn 	= leafDataToChar( deviceCfgFile,_name,"Command","InduceOn" );
	_induceOff	= leafDataToChar( deviceCfgFile,_name,"Command","InduceOff" );
	_setInduc	= leafDataToChar( deviceCfgFile,_name,"Command","SetInduc" );
	_getInduc	= leafDataToChar( deviceCfgFile,_name,"Command","GetInduc" );

	jdebug("End of ProCounter's loadOwnConst");
	if(_activeOn == 0) return FAIL;
	return SUCCESS;
}

void ProCounter::setName(const char* name) {
	_identName = new char(strlen(name));
	_identSection = new char(strlen(name));
	strcpy((char*)_identName, name);
	strcpy((char*)_identSection, name);
}

void ProCounter::addBlockLabel(int newLabel) {
	if(blockLabel.size() <= (uint)_nBlockLabel) {
		blockLabel.resize(blockLabel.size() + 100);
	}
	blockLabel[_nBlockLabel] = newLabel;
	_nBlockLabel++;
}

int ProCounter::stop(int dialog) {
	jdebug("counter::stop");
	return sendCommand(_stop, dialog);
}

int ProCounter::setMasterOn(int dialog) {
	jdebug("counter::setMasterOn");
	return sendCommand(_masterOn, dialog);
}

int ProCounter::setMasterOff(int dialog) {
	jdebug("counter::setMasterOff");
	return sendCommand(_masterOff, dialog);
}

int ProCounter::setActiveOn(int dialog) {
	jdebug("counter::setActiveOn");
	return sendCommand(_activeOn, dialog);
}

int ProCounter::setActiveOff(int dialog) {
	jdebug("counter::setActiveOff");
	return sendCommand(_activeOff, dialog);
}

int ProCounter::setInduceOn(int dialog) {
	jdebug("counter::setInduceOn");
	return sendCommand(_induceOn, dialog);
}

int ProCounter::setInduceOff(int dialog) {
	jdebug("counter::setInduceOff");
	return sendCommand(_induceOff, dialog);
}

int ProCounter::setInduc(uchar masterAddress, int dialog) {
	jdebug("counter::setInduc");
	int current = UNDEFINED;
	return setValue(masterAddress, current, _setInduc, _getInduc,
						"Error in setInduc!", dialog);
}

int ProCounter::setBlSize(uchar blockSize, int dialog) {
	jdebug("counter::setBlSize");
	if( (blockSize > MAX_BLOCK_SIZE) || (blockSize < MIN_BLOCK_SIZE) ) {
		const char* errText = "Invalid block size!";
		reportError(_identName, errText, dialog);
		return FAIL;
	}
	return setValue(blockSize, _blockSize, _setBlSize, _getBlSize,
						"Error in setBlSize!", dialog);
}

int ProCounter::setNumber(int expositionNumber, int dialog) {
	jdebug("counter::setNumber");
	if( (expositionNumber > MAX_EXPOSITION_NUMBER) ||
			(expositionNumber < MIN_EXPOSITION_NUMBER) ) {
		const char* errText = "Invalid exposition number!";
		reportError(_identName, errText, dialog);
		return FAIL;
	}
	resizeData(expositionNumber);
	return setValue(expositionNumber,_expositionNumber,_setNumber,
						_getNumber, "Error in getNumber!", dialog);
}

int ProCounter::setExpos(double expos, int dialog) {
	jdebug("counter::setExpos");

	if( (expos < MIN_EXPOSE ) || (expos > MAX_EXPOSE ) ) {
		const char* errText = "Invalid exposure!";
		reportError(_identName, errText, dialog);
		return FAIL;
	}
	int exposValue = int(rint(expos*(_const3+(_const4<<8) - 1.0) / 8.0));

	return setValue(exposValue,_exposValue,_setExpos,_getExpos,
					"Error in setExpos!", dialog);
}

int ProCounter::runMst(int dialog) {
	jdebug("counter::runMst");
	return sendCommand(_runMst, dialog);
}

int ProCounter::runTest(int dialog) {
	jdebug("counter::runTest");
	return sendCommand(_runTest, dialog);
}
