/*--------------------------------------------------------------------------*/
/** @file trbexport.h
@brief  The class is presented with static fields which serve for exchange of 
data between the Scenario thread and Main-thread of the program.

                  MASS project: TURBINA module file
                       header file for trbexport.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef TRBEXPORT_H
#define TRBEXPORT_H

#include <string>
#include <qstring.h>

struct DeviceState {
	DeviceState(void) {hv=-1;illum=-1;light=-1;safety=-1;
#ifdef __MOTOR__
			door=-1;aperture=-1;
#endif
#ifdef __MARC_CONFIG__
			buttons=-1;
#else
			mirror=-1;
#endif
			temperature=99.9;brightness=-1; }

	int	hv;		// on/off
	int	illum;		// on/off
	int	light;		// on/off
	int	safety;		// on/off
#ifdef __MOTOR__
	int	door;		// on/off (open/close)
	int	aperture;	// number
#endif
#ifdef __MARC_CONFIG__
	int	buttons;	// unlock/lock
#else
	int mirror;		// on/off
#endif
	double	temperature;
	int	brightness;	// of illumination
};

struct Progress {
	QString	message;
	double	progress;
};

const int MAX_ROW=40;
const int MAX_COL=7;

class ExportText {
public:
	static QString text;
	static bool isChanged;
};

class ExportTable {
private:
	static QString table[MAX_ROW][MAX_COL];
public:
	static bool isChanged;
	static bool clean;
	static bool adjust;
	static int startRow,nRows;
	static int startCol,nCols;

	ExportTable(void);
	~ExportTable();

	void setNum(int row, int col, double val, const char* format);
	void setNum(int row, int col, int val, const char* format);
	void setHHead(int,const char*,const char* = NULL,
		const char* = NULL, const char* = NULL,
		const char* = NULL, const char* = NULL,
		const char* = NULL, const char* = NULL,
					const char* = NULL);
	void cleanRow(int);
	void setText(int row, int col, const char* text);
	QString& getData(int row, int col);
};

class InfoExport {
public:
	static bool	isProgressChanged;
	static Progress	progress;
	static bool	isModeChanged;
	static Progress	mode;
	static bool	isScenarioChanged;
	static Progress	scenario;
	static bool	isScenario;
	static bool	isScenarioStateChanged;
	static bool	isMessage;
	static QString	message;
	static int	ms;
	static bool	isStarPositionChanged;
	static int	starPosition;
};

namespace StateExport {

	void exportError(QString& mess);
	void cleanError(void);
	bool error(void);
	QString message(void);
	void setCorrectionReady(bool ready);
	void cleanCorrectionReady(void);

	void startUserTrack(int coordinate, int direction);
	void stopUserTrack(void);
	bool isUserTrack(void);
	int trackDirection(void);
	int trackCoordinate(void);

	bool correction(void);
	int hv(void);
#ifdef __MOTOR__
	int door(void);
	int aperture(void);
#endif
#ifdef __MARC_CONFIG__
	int buttons(void);
#else
	int mirror(void);
#endif
	int illum(void);
	int light(void);
	int safety(void);
	double temperature(void);
	int brightness(void);

	bool isStartCounting(void);
	bool isEndCounting(void);

	bool isWithDevice(void);
	bool isWithoutDevice(void);
	bool isChanged(void);

#ifdef __MOTOR__
	void setAperture(int aperture);
	void setDoor(int open);
#endif
#ifdef __MARC_CONFIG__
	void setButtons(int on);
#else
	void setMirror(int on);
#endif
	void setHV(int on);
	void setIllum(int on);
	void setLight(int on);
	void setSafety(int on);
	void setTemperature(double value);
	void setBrightness(int value);

	void setStartCounting(void);
	void cleanStartCounting(void);
	void setEndCounting(void);
	void cleanEndCounting(void);
	void setWithDevice(void);
	void setWithoutDevice(void);
	void cleanWithDevice(void);
	void cleanWithoutDevice(void);
	void cleanChanged(void);
};

namespace SVexport {
	void reset(void);
	int set(std::string& instruction);
	int get(std::string& instruction);
};

/*
namespace SVexport {

enum instruction {INIT, PARK, PLAY, RUN, STOP, 
									BREAK, QUIT, NEW_OBJECT, LAST};

enum status {READY, PARKED, ERSYN, ERFAT, ERANG, BUSY, NOSTAR};


	void	reset(void);

	const char* getErStatus(status erstat);

	bool	isChanged(void);
	void	cleanChanged(void);

	bool	set(int comid, instruction what);
	bool	set(int comid, std::string& what);
	void	clean(instruction what);
	int		get(instruction what);

	void	setNewStar(int comid, const char* hr);
	int		getNewStar(const char** hr);
	void	setStatus(status stat);
	const char*	getStatus(void);
};
*/

#endif
