#include <qstring.h>
#include <math.h>
#include "trbtime.h"
#include "trbconst.h"
#include "trberror.h"
#include "cfg.h"
#include "messages.h"
#include "tools.h"
// To prevent problems with missing built-in exit():
#include <stdlib.h>

extern FILE* out_file;
extern CFGTree* tree;

Time::~Time() {
}

QString Time::toString( void ) {
	QString str;
	str.sprintf( "%02d:%02d:%02d",_hour,_minute,_second );
	return str;
}

long Time::allSec( void ) {
	long s = long(_second) + 
		(long(_minute) + long(_hour)*60)*60;
	return s;
}
void Time::setToValidInterval( long sec ) {
	setToValidInterval( double(sec) );
}

void Time::setToValidInterval( double sec ) {
	sec = rint( sec );
	double val = sec / 3600.0;
	_hour = int( floor(val) );
	_minute = int( floor( (val-_hour)*60.0 ) );
	_second = int( rint(((val-_hour)*60.0-_minute)*60.0) );
//	debug( "toValidTime: s=%f -> %02d:%02d:%02d",
//		sec,_hour,_minute,_second );
}

void Time::addSecs( int s ) {
	setToValidInterval( allSec() + s );
}

int Time::secsTo( Time& t ) {
	long sec = t.allSec() - allSec();
	return int(sec);
}

void ForSlot::change( void ) {
	jdebug( "trbtime ForSlot: slot change!!!" );
	trb_tm::loadConst( false );
}

namespace trb_tm {
	Time		_currentPlayBackUT;
	QDateTime	_currentPlayBackDateTime;
	bool		_isPlayBack;	
	
	Time	_programStartTime;
	QDate	_programStartDate;
	double	_tmZone;
	double	_longitude;	// h_sec
	double	_sinLatitude;
	double	_cosLatitude;
	ForSlot	forSlot;
}

void trb_tm::enablePlayBack( bool isPB ) {
	_isPlayBack = isPB;
}

bool trb_tm::setPlayBackDate( int y, int m, int d ) {
	QDate date;
	bool valid = date.setYMD(y,m,d);
	if( !valid ) return( false );
	_currentPlayBackDateTime.setDate( date );
	return( true );
}

void trb_tm::setPlayBackTime( int h, int m, int s ) {
	_currentPlayBackUT.setHMS(h,m,s);
}

void trb_tm::setStartTime( int h, int m, int s ) {
	_programStartTime.setHMS(h,m,s);
//	debug( "setStartTime: h=%d m=%d s=%d: -> _programStartTime: %02d %02d %02d",
//	_programStartTime.hour(),_programStartTime.minute(),
//		_programStartTime.second(),h,m,s );
}

void trb_tm::setStartTime( Time time ) {
	_programStartTime = time;
//	debug( "_programStartTime = time-> _programStartTime: %02d %02d %02d"
//	"\ntime:%02d %02d %02d",
//		_programStartTime.hour(),_programStartTime.minute(),
//		_programStartTime.second(),time.hour(),
//		time.minute(),time.second() );
}

Time trb_tm::programStartTime( void ) {
	return _programStartTime;
}

QDate trb_tm::programStartDate( void ) {
	return _programStartDate;
}

long trb_tm::longDate( QDate date ) {
	long ldate = date.day()*10000 +
		date.month()*100 + date.year() - 2000;
	return ldate;
}

double trb_tm::tmZone( void ) {
	return _tmZone;
}

double trb_tm::sinLatitude( void ) {
	return _sinLatitude;
}

double trb_tm::cosLatitude( void ) {
	return _cosLatitude;
}

void trb_tm::loadConst( bool isFirst) {
	jdebug( "trbtime::loadConst   start..." );

	_longitude = 0;
	double latitude = 0;
	_tmZone = leafDataToDouble( cfgFile,"General",
		"Site", "TimeZone" );
	try {
		_longitude = leafDataToTimeSec( cfgFile,"General",
			"Site","Longitude" );
		latitude = leafDataToDegSec( cfgFile,"General",
			"Site","Latitude" );

// Tut v out_file ne vyvodim, t.k. oni yecho ne gotovy
	} catch( LoadError err ) {
		QString str_message = cfgFile;
		str_message.append( "\\General\\Site\\: " );
		str_message.append( err.getMessage() );
		showFatalError( str_message );
		exit(1);
	}
	latitude *= RAD_IN_ASEC;
	_sinLatitude = sin( latitude );
	_cosLatitude = cos( latitude );

	if( isFirst ) {
		_programStartTime = QTime::currentTime();

//		debug( "START: -> _programStartTime: %02d %02d %02d",
//			_programStartTime.hour(),
//			_programStartTime.minute(),
//			_programStartTime.second() );

		_programStartDate = QDate::currentDate();
		if( _programStartTime.hour() < HOUR_DATE_CHANG ) {
			// local'nyi vecher byl vchera
			_programStartDate = _programStartDate.addDays(-1);
		}
		_programStartTime
			.addSecs( int(-1.0*_tmZone*SEC_IN_HOUR) );
		QObject::connect( tree, SIGNAL(changed()), &forSlot, SLOT(change()) );
	}
	jdebug( "trbtime::loadConst   finish..." );
}

Time trb_tm::getCurrentUT( void ) {		// to UT
	Time time;
	if( _isPlayBack ) {
		time = _currentPlayBackUT;
	} else {
		QTime ut =  QTime::currentTime()
			.addSecs( int(-1.0*trb_tm::tmZone()*SEC_IN_HOUR) );
		int hour = ut.hour();
		time = ut;
		if( (hour+_tmZone) < HOUR_DATE_CHANG ) {
			hour += HOUR_IN_DAY;
			time.setHour(hour);
		}
	}
	return time;
}

double trb_tm::toSec( QTime tm ) {
	double sec = toSec( tm.hour(),tm.minute(),tm.second() );
	return sec;
}

double trb_tm::toSec( Time tm ) {
	double sec = toSec( tm.hour(),tm.minute(),tm.second() );
	return sec;
}

double trb_tm::toSec( int h, int m, int s ) {
        double sec = ( (double)h*60.0 + (double)m )*60.0 + (double)s;
        return sec;
}

long trb_tm::julian( QDate utDate ) {

/* Calculation of the Julian date */

	int month = utDate.month();
	int day = utDate.day();
	int year = utDate.year();

	long result = (3057*(long)month)/100 +
		(365*(long)year) + 1721028L;
	result = result + (long)(year/4 - year/100 + year/400 + day);
	if( month > 2 ) return( result);
	int j = 2;
	if( year%4 == 0 )       j -= 1;
	if( year%100 == 0 )     j -= 1;
	if( year%400 == 0 )     j += 1;
	result += j;
	return result;
}

double trb_tm::calcS0Time( QDate utDate ) {

/* Calculation of siderial time in the Greewhich midnigth */

	double c3	= 1.0775925925925926E-06;
	double c2	= 2.7379093507870371E-03;
	double c1	= 0.2790572732638889;

	double d = (double)julian( utDate ) - (double)2451545.5;
	double t2 = d/(double)36525.0;
	double result = c1 + c2*d + c3*t2*t2;
	result = result - floor( result );   
	return result;
}

double trb_tm::calcLocalST( void ) {

/* Calculation of Current Local Sideral Time */

	QDateTime utDateTime;
	if( _isPlayBack ) {
		utDateTime = _currentPlayBackDateTime;
		utDateTime = utDateTime.addSecs(int(rint(toSec(_currentPlayBackUT))));
	} else {
		utDateTime = QDateTime::currentDateTime();
		utDateTime = utDateTime.addSecs( int(-1.0*_tmZone*SEC_IN_HOUR) ); //to UT
	}
	double sec = toSec( utDateTime.time() );
	double st = calcS0Time( utDateTime.date() ) 
		+ MU*(double)sec / SEC_IN_DAY;
	st = SEC_IN_DAY*( st - floor( st ) );
	double stLoc = st + _longitude;
	return stLoc;
}
